using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Reflection;
using LighthouseShared;

namespace Lighthouse
{
    public partial class FormChangeGeneralPlugins : Form
    {
        FormMain m_Parent;

        public FormChangeGeneralPlugins(FormMain parent)
        {
            m_Parent = parent;

            InitializeComponent();

            foreach (string s in m_Parent.m_GeneralPlugins_Filenames)
            {
                Assembly asm = Assembly.LoadFile(Directory.GetCurrentDirectory() + @"\Plugins\" + s);

                foreach (Type t in asm.GetTypes())
                {
                    if (t.GetInterface(typeof(IGeneralPlugin).Name) != null)
                    {

                        PluginListItem item = new PluginListItem(s, t.Name);


                        if (parent.m_GeneralPlugins_Filenames_Loaded.Contains(item.Path))
                        {
                            checkedListBox1.Items.Add(item, true);
                        }
                        else
                        {
                            checkedListBox1.Items.Add(item, false);
                        }
                    }
                }
            }


            foreach (string s in Directory.GetFiles(Directory.GetCurrentDirectory() + @"\Plugins\", "*.dll"))
            {
                if (!m_Parent.m_GeneralPlugins_Filenames.Contains(Path.GetFileName(s)))
                {

                    Assembly asm = Assembly.LoadFile(s);

                    foreach (Type t in asm.GetTypes())
                    {
                        if (t.GetInterface(typeof(IGeneralPlugin).Name) != null)
                        {

                            PluginListItem item = new PluginListItem(Path.GetFileName(s), t.Name);

                            checkedListBox1.Items.Add(item, false);
                        }
                    }
                }
            }
        }

        private void FormChangeGeneralPlugins_Load(object sender, EventArgs e)
        {

        }

        private void checkedListBox1_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                checkedListBox1.SelectedIndex = checkedListBox1.IndexFromPoint(e.Location);
            }
        }

        private void moveUpToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                bool ischecked = checkedListBox1.GetItemChecked(checkedListBox1.SelectedIndex);
                int index = checkedListBox1.SelectedIndex - 1;

                if (index >= 0)
                {
                    object item = checkedListBox1.SelectedItem;

                    checkedListBox1.Items.Remove(item);
                    checkedListBox1.Items.Insert(index, item);

                    checkedListBox1.SelectedIndex = index;

                    checkedListBox1.SetItemChecked(index, ischecked);
                }
            }
            catch (Exception)
            {
            }
        }

        private void moveDownToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                bool ischecked = checkedListBox1.GetItemChecked(checkedListBox1.SelectedIndex);
                int index = checkedListBox1.SelectedIndex + 1;

                if (index < checkedListBox1.Items.Count)
                {
                    object item = checkedListBox1.SelectedItem;

                    checkedListBox1.Items.Remove(item);
                    checkedListBox1.Items.Insert(index, item);

                    checkedListBox1.SelectedIndex = index;

                    checkedListBox1.SetItemChecked(index, ischecked);
                }
            }
            catch (Exception)
            {
            }
        }

        private void moveToTopToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                bool ischecked = checkedListBox1.GetItemChecked(checkedListBox1.SelectedIndex);
                object item = checkedListBox1.SelectedItem;

                checkedListBox1.Items.Remove(item);
                checkedListBox1.Items.Insert(0, item);

                checkedListBox1.SelectedIndex = 0;

                checkedListBox1.SetItemChecked(0, ischecked);
            }
            catch (Exception)
            {
            }

        }

        private void moveToBottomToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                bool ischecked = checkedListBox1.GetItemChecked(checkedListBox1.SelectedIndex);
                object item = checkedListBox1.SelectedItem;

                checkedListBox1.Items.Remove(item);
                checkedListBox1.Items.Add(item);

                checkedListBox1.SelectedIndex = checkedListBox1.Items.Count - 1;

                checkedListBox1.SetItemChecked(checkedListBox1.SelectedIndex, ischecked);
            }
            catch (Exception)
            {
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            DialogResult result = MessageBox.Show("Are you sure you want to apply these changes?\nAll plugins in the list will be reloaded.", "Question", MessageBoxButtons.YesNo, MessageBoxIcon.Question);

            if (result == DialogResult.Yes)
            {
                m_Parent.UnloadGeneralPlugins();

                m_Parent.m_GeneralPlugins_Filenames.Clear();
                m_Parent.m_GeneralPlugins_Filenames_Loaded.Clear();

                foreach (object o in checkedListBox1.Items)
                {
                    PluginListItem item = (PluginListItem)o;

                    m_Parent.m_GeneralPlugins_Filenames.Add(item.Path);
                }

                foreach (object o in checkedListBox1.CheckedItems)
                {
                    PluginListItem item = (PluginListItem)o;

                    m_Parent.m_GeneralPlugins_Filenames_Loaded.Add(item.Path);
                }

                m_Parent.LoadGeneralPlugins();

                this.Close();
            }
        }

        private class PluginListItem
        {
            public string Path;
            public string Name;

            public PluginListItem(string path, string name)
            {
                Path = path;
                Name = name;
            }

            public override string ToString()
            {
                return this.Name;
            }
        }

        private void button2_Click(object sender, EventArgs e)
        {
            this.Close();
        }
    }
}