using System;
using System.Collections.Generic;
using System.Text;

namespace LighthouseShared
{
    public delegate void ChatJoinHandler(IClientPlugin sender, IChatInfo chatinfo);
    public delegate void ChatLeaveHandler(IClientPlugin sender, IChatInfo chatinfo, bool banned, bool closed);
    public delegate void ChatMessageHandler(IClientPlugin sender, IChatInfo chatinfo, string screenname, ulong icbmcookie, string message);
    public delegate void ChatUserHandler(IClientPlugin sender, IChatInfo chatinfo, string screenname);
    public delegate bool RawPacketHandler(IClientPlugin sender, byte[] packet);
    public delegate void StatusUpdateHandler(IClientPlugin sender, string status);
    public delegate void SignOnHandler(IClientPlugin sender);
    public delegate void SignOffHandler(IClientPlugin sender);
    public delegate void ErrorCodeHandler(IClientPlugin sender, int code, string description);

    public interface IClientPlugin
    {
        bool SignedOn { get; }
        string ScreenName { get; }

        event ChatJoinHandler OnChatJoin;
        event ChatLeaveHandler OnChatLeave;
        event ChatMessageHandler OnChatMessage;
        event ChatUserHandler OnChatOwnerDetect;
        event ChatUserHandler OnChatBan;
        event ChatUserHandler OnChatBanFailed;
        event ChatUserHandler OnChatUnban;
        event ChatUserHandler OnChatUnbanFailed;
        event ChatUserHandler OnChatKick;
        event ChatUserHandler OnChatKickFailed;
        event ChatUserHandler OnChatUserJoin;
        event ChatUserHandler OnChatUserLeave;

        event StatusUpdateHandler OnStatusUpdate;

        event SignOnHandler OnSignedOn;
        event SignOffHandler OnSignedOff;
        event ErrorCodeHandler OnSignOnError;

        event RawPacketHandler OnRawPacket;

        void SignOn(string screenname, string password);
        void SignOff();
        void ChatJoin(string roomname, ushort exchange);
        void ChatLeave(IChatInfo chatinfo);
        ulong ChatMessage(IChatInfo chatinfo, string message);
        void ChatBan(IChatInfo chatinfo, string screenname);
        void ChatUnban(IChatInfo chatinfo, string screenname);
        void ChatKick(IChatInfo chatinfo, string screenname);
        void ChatClose(IChatInfo chatinfo);
    }
}
