using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;

namespace Oscar5Plugin
{
    struct PacketBuilder
    {
        public static FLAPPacket FLAPConnection()
        {
            FLAPPacket packet = new FLAPPacket();
            packet.Data = new byte[4];
            packet.Data[3] = 0x01;
            packet.Channel = FLAPChannel.Connect;
            return packet;
        }

        public static FLAPPacket FLAPCookie(byte[] cookie)
        {
            FLAPPacket packet = new FLAPPacket();
            packet.Data = new byte[8 + cookie.Length];
            packet.Data[3] = 0x01;
            Tools.PutWord(packet.Data, 4, 0x0006);
            Tools.PutWord(packet.Data, 6, (ushort)cookie.Length);
            Buffer.BlockCopy(cookie, 0, packet.Data, 8, cookie.Length);
            packet.Channel = FLAPChannel.Connect;
            return packet;
        }

        public static FLAPPacket FLAPClose()
        {
            FLAPPacket packet = new FLAPPacket();
            packet.Data = new byte[0];
            packet.Channel = FLAPChannel.Disconnect;
            return packet;

        }

        public struct SNACGeneric
        {
            public static FLAPPacket Capabilities(FLAPSocketType sockettype, bool allowchat, bool allowim)
            {
                byte[] caps = null;

                switch (sockettype)
                {
                    case FLAPSocketType.BOS:
                        if (allowim)
                        {
                            caps = new byte[] { 0x00, 0x01, 0x00, 0x04, 0x00, 0x13, 0x00, 0x03, 
                                            0x00, 0x02, 0x00, 0x01, 0x00, 0x03, 0x00, 0x01,
                                            0x00, 0x04, 0x00, 0x01, 0x00, 0x06, 0x00, 0x01,
                                            0x00, 0x08, 0x00, 0x01, 0x00, 0x09, 0x00, 0x01,
                                            0x00, 0x0a, 0x00, 0x01, 0x00, 0x0b, 0x00, 0x01
                                           };
                        }
                        else
                        {
                            caps = new byte[] { 0x00, 0x01, 0x00, 0x04, 0x00, 0x13, 0x00, 0x03, 
                                            0x00, 0x02, 0x00, 0x01, 0x00, 0x03, 0x00, 0x01,
                                             0x00, 0x06, 0x00, 0x01,
                                            0x00, 0x08, 0x00, 0x01, 0x00, 0x09, 0x00, 0x01,
                                            0x00, 0x0a, 0x00, 0x01, 0x00, 0x0b, 0x00, 0x01
                                           };
                        }

                        break;
                    case FLAPSocketType.ChatNavigation:
                        caps = new byte[] { 0x00, 0x01, 0x00, 0x04, 0x00, 0x0d, 0x00, 0x01 };
                        break;

                    case FLAPSocketType.Chat:
                        if (allowchat)
                            caps = new byte[] { 0x00, 0x01, 0x00, 0x04, 0x00, 0x0e, 0x00, 0x01 };
                        else
                            caps = new byte[] { 0x00, 0x01, 0x00, 0x04};
                        break;
                                            
                }

                FLAPPacket packet = new FLAPPacket();
                packet.Data = new byte[10 + caps.Length];
                Tools.PutWord(packet.Data, 0, 0x0001);
                Tools.PutWord(packet.Data, 2, 0x0017);
                Buffer.BlockCopy(caps, 0, packet.Data, 10, caps.Length);
                return packet;
            }

            public static FLAPPacket RateInfoRequest()
            {
                FLAPPacket packet = new FLAPPacket();
                packet.Data = new byte[10];
                Tools.PutWord(packet.Data, 0, 0x0001);
                Tools.PutWord(packet.Data, 2, 0x0006);
                return packet;
            }

            public static FLAPPacket RateInfoAck(byte[] rateclasses)
            {
                FLAPPacket packet = new FLAPPacket();
                packet.Data = new byte[10 + rateclasses.Length];
                Tools.PutWord(packet.Data, 0, 0x0001);
                Tools.PutWord(packet.Data, 2, 0x0008);
                Buffer.BlockCopy(rateclasses, 0, packet.Data, 10, rateclasses.Length);
                return packet;
            }

            public static FLAPPacket ClientReady(FLAPSocketType sockettype, bool allowchat, bool allowim)
            {
                byte[] services = null;

                switch (sockettype)
                {
                    case FLAPSocketType.BOS:
                        if (allowim)
                        {
                            services = new byte[] { 0x00, 0x01, 0x00, 0x04, 0x01, 0x10, 0x08, 0xd6, 
                                                0x00, 0x13, 0x00, 0x03, 0x01, 0x10, 0x08, 0xd6, 
                                                0x00, 0x02, 0x00, 0x01, 0x01, 0x10, 0x08, 0xd6, 
                                                0x00, 0x03, 0x00, 0x01, 0x01, 0x10, 0x08, 0xd6, 
                                                0x00, 0x04, 0x00, 0x01, 0x01, 0x10, 0x08, 0xd6, 
                                                0x00, 0x06, 0x00, 0x01, 0x01, 0x10, 0x08, 0xd6, 
                                                0x00, 0x08, 0x00, 0x01, 0x01, 0x04, 0x00, 0x01, 
                                                0x00, 0x09, 0x00, 0x01, 0x01, 0x10, 0x08, 0xd6, 
                                                0x00, 0x0a, 0x00, 0x01, 0x01, 0x10, 0x08, 0xd6, 
                                                0x00, 0x0b, 0x00, 0x01, 0x01, 0x10, 0x08, 0xd6};
                        }
                        else
                        {
                            services = new byte[] { 0x00, 0x01, 0x00, 0x04, 0x01, 0x10, 0x08, 0xd6, 
                                                0x00, 0x13, 0x00, 0x03, 0x01, 0x10, 0x08, 0xd6, 
                                                0x00, 0x02, 0x00, 0x01, 0x01, 0x10, 0x08, 0xd6, 
                                                0x00, 0x03, 0x00, 0x01, 0x01, 0x10, 0x08, 0xd6, 
                                                0x00, 0x06, 0x00, 0x01, 0x01, 0x10, 0x08, 0xd6, 
                                                0x00, 0x08, 0x00, 0x01, 0x01, 0x04, 0x00, 0x01, 
                                                0x00, 0x09, 0x00, 0x01, 0x01, 0x10, 0x08, 0xd6, 
                                                0x00, 0x0a, 0x00, 0x01, 0x01, 0x10, 0x08, 0xd6, 
                                                0x00, 0x0b, 0x00, 0x01, 0x01, 0x10, 0x08, 0xd6};
                        }
                        break;
                    case FLAPSocketType.ChatNavigation:
                        services = new byte[] { 0x00, 0x01, 0x00, 0x04, 0x00, 0x0d, 0x00, 0x01 };
                        break;
                    case FLAPSocketType.Chat:
                        if (allowchat)
                        {
                            services = new byte[] { 0x00, 0x01, 0x00, 0x04, 0x00, 0x10, 0x08, 0xd6, 
                                                0x00, 0x0e, 0x00, 0x01, 0x00, 0x10, 0x08, 0xd6 };
                        }
                        else
                        {
                            services = new byte[] { 0x00, 0x01, 0x00, 0x04, 0x00, 0x10, 0x08, 0xd6, 
                                                };
                        }
                        break;
                }

                FLAPPacket packet = new FLAPPacket();
                packet.Data = new byte[10 + services.Length];
                Tools.PutWord(packet.Data, 0, 0x0001);
                Tools.PutWord(packet.Data, 2, 0x0002);
                Buffer.BlockCopy(services, 0, packet.Data, 10, services.Length);
                return packet;
            }


            public static FLAPPacket ServiceRequest(ushort servicetype)
            {
                FLAPPacket packet = new FLAPPacket();
                packet.Data = new byte[12];
                Tools.PutWord(packet.Data, 0, 0x0001);
                Tools.PutWord(packet.Data, 2, 0x0004);
                Tools.PutWord(packet.Data, 10, servicetype);
                return packet;
            }

            public static FLAPPacket ChatServiceRequest(ChatInfo chatinfo, uint fnacid)
            {
                FLAPPacket packet = new FLAPPacket();
                packet.Data = new byte[10 + 9 + chatinfo.FullName.Length + 2];
                Tools.PutWord(packet.Data, 0, 0x0001);
                Tools.PutWord(packet.Data, 2, 0x0004);
                Tools.PutDword(packet.Data, 6, fnacid);
                Tools.PutWord(packet.Data, 10, 0x000e);
                Tools.PutWord(packet.Data, 12, 0x0001);
                Tools.PutWord(packet.Data, 14, (ushort)(chatinfo.FullName.Length + 5));
                Tools.PutWord(packet.Data, 16, 0x0004);
                packet.Data[18] = (byte)(chatinfo.FullName.Length);
                Encoding.ASCII.GetBytes(chatinfo.FullName, 0, chatinfo.FullName.Length, packet.Data, 19);
                return packet;
            }
        }

        public struct SNACSignOn
        {
            public static FLAPPacket SignOn(string screenname)
            {
                FLAPPacket packet = new FLAPPacket();
                packet.Data = new byte[10 + 4 + screenname.Length];
                Tools.PutWord(packet.Data, 0, 0x0017);
                Tools.PutWord(packet.Data, 2, 0x0006);

                Tools.PutWord(packet.Data, 10, 0x0001);
                Tools.PutWord(packet.Data, 12, (ushort)screenname.Length);
                Encoding.ASCII.GetBytes(screenname, 0, screenname.Length, packet.Data, 14);

                return packet;
            }

            public static FLAPPacket Logon(string screenname, string password, byte[] challenge)
            {
                List<byte> tlvs = new List<byte>();
                tlvs.AddRange(Tools.MakeTLV(0x0001, screenname));
                tlvs.AddRange(new byte[] { 0x00, 0x25, 0x00, 0x10 });

                System.Security.Cryptography.MD5 md5 = System.Security.Cryptography.MD5.Create();
                //tmpPasswordHash = DeHex(UCase(CalculateMD5(tmpChallenge & DeHex(CalculateMD5(m_Password)) & "AOL Instant Messenger (SM)")))
                // big fag juicy cock from vb6's butthole^^
                byte[] md5pw = md5.ComputeHash(Encoding.ASCII.GetBytes(password));
                byte[] md5salt = Encoding.ASCII.GetBytes("AOL Instant Messenger (SM)");

                byte[] md5tmp = new byte[challenge.Length + md5pw.Length + md5salt.Length];
                Buffer.BlockCopy(challenge, 0, md5tmp, 0, challenge.Length);
                Buffer.BlockCopy(md5pw, 0, md5tmp, challenge.Length, md5pw.Length);
                Buffer.BlockCopy(md5salt, 0, md5tmp, challenge.Length + md5pw.Length, md5salt.Length);

                byte[] md5hash = md5.ComputeHash(md5tmp);

                tlvs.AddRange(md5hash);
                tlvs.AddRange(new byte[] { 0x00, 0x4c, 0x00, 0x00 });
                tlvs.AddRange(Tools.MakeTLV(0x0003, "AOL Instant Messenger, version 5.5.3572/WIN32"));

                // i duno what the bloody fuck this shit is but it looked good at the time
                tlvs.AddRange(new byte[] { 0x00, 0x16, 0x00, 0x02, 0x01, 0x09 });
                tlvs.AddRange(new byte[] { 0x00, 0x17, 0x00, 0x02, 0x00, 0x05 });
                tlvs.AddRange(new byte[] { 0x00, 0x18, 0x00, 0x02, 0x00, 0x05 });
                tlvs.AddRange(new byte[] { 0x00, 0x19, 0x00, 0x02, 0x00, 0x00 });
                tlvs.AddRange(new byte[] { 0x00, 0x1a, 0x00, 0x02, 0x0d, 0xf4 });
                tlvs.AddRange(new byte[] { 0x00, 0x14, 0x00, 0x04, 0x00, 0x00, 0x00, 0x00 });
                tlvs.AddRange(new byte[] { 0x00, 0x0f, 0x00, 0x02, 0x65, 0x6e });
                tlvs.AddRange(new byte[] { 0x00, 0x0e, 0x00, 0x02, 0x75, 0x73 });
                tlvs.AddRange(new byte[] { 0x00, 0x4a, 0x00, 0x01, 0x01 });

                byte[] tlvdata = tlvs.ToArray();

                FLAPPacket packet = new FLAPPacket();
                packet.Data = new byte[10 + tlvdata.Length];
                Tools.PutSNACHeader(packet.Data, 0x0017, 0x0002, 0, 0);
                Buffer.BlockCopy(tlvdata, 0, packet.Data, 10, tlvdata.Length);

                return packet;
            }
        }

        public struct SNACChatNav
        {
            public static FLAPPacket Create(ChatInfo chatinfo, uint fnacid)
            {
                FLAPPacket packet = new FLAPPacket();
                packet.Data = new byte[10 + 2 + 1 + 6 + 2 + 1 + 2 + 2 + 2 + 8 + 4 + chatinfo.Name.Length + 4 + 2];

                Tools.PutWord(packet.Data, 0, 0x000d);
                Tools.PutWord(packet.Data, 2, 0x0008);

                Tools.PutDword(packet.Data, 6, fnacid);

                int offset = 10;

                Tools.PutWord(packet.Data, offset, chatinfo.Exchange);
                offset += 2;
                packet.Data[offset] = 0x06;
                offset += 2;
                Encoding.ASCII.GetBytes("create", 0, 6, packet.Data, offset);
                offset += 6;
                packet.Data[offset] = 0xff;
                offset += 1;
                packet.Data[offset] = 0xff;
                offset += 2;
                packet.Data[offset] = 0x03;
                offset += 1;
                Tools.PutWord(packet.Data, offset, 0x00d6);
                offset += 2;
                Tools.PutWord(packet.Data, offset, 0x0008);
                offset += 2;
                Encoding.ASCII.GetBytes("us-ascii", 0, 8, packet.Data, offset);
                offset += 8;
                Tools.PutWord(packet.Data, offset, 0x00d3);
                offset += 2;
                Tools.PutWord(packet.Data, offset, (ushort)chatinfo.Name.Length);
                offset += 2;
                Encoding.ASCII.GetBytes(chatinfo.Name, 0, chatinfo.Name.Length, packet.Data, offset);
                offset += chatinfo.Name.Length;
                Tools.PutWord(packet.Data, offset, 0x00d7);
                offset += 2;
                Tools.PutWord(packet.Data, offset, 0x0002);
                offset += 2;
                //you know i could just add these two bytes manually isntead of decoding a string but what the hell im bored
                Encoding.ASCII.GetBytes("en", 0, 2, packet.Data, offset);
                offset += 2;

                return packet;
            }
        }

        public struct SNACChat
        {
            public static FLAPPacket Message(string message, ulong icbmcookie)
            {
                byte[] cookie = new byte[8];

                Tools.PutQword(cookie, 0, icbmcookie);
                Console.WriteLine("sent cookie: {0}", icbmcookie);

                List<byte> inner = new List<byte>();
                inner.AddRange(new byte[] { 0x00, 0x05, 0x00, 0x00 });
                inner.AddRange(Tools.MakeTLV(0x0004, "text/x-aolrtf"));
                inner.AddRange(Tools.MakeTLV(0x0002, "us-ascii"));
                inner.AddRange(Tools.MakeTLV(0x0003, "en"));
                inner.AddRange(Tools.MakeTLV(0x0005, "binary"));
                inner.AddRange(Tools.MakeTLV(0x0001, message));
                Tools.PutWord(inner, 2, (ushort)(inner.Count - 4));

                List<byte> outer = new List<byte>();
                outer.AddRange(new byte[] { 0x00, 0x0e, 0x00, 0x05, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 });
                outer.AddRange(cookie); //munchmunchyum
                outer.AddRange(new byte[] { 0x00, 0x003 });
                outer.AddRange(new byte[] { 0x00, 0x01, 0x00, 0x00 });
                outer.AddRange(new byte[] { 0x00, 0x06, 0x00, 0x00 });
                outer.AddRange(inner);

                FLAPPacket packet = new FLAPPacket();
                packet.Data = outer.ToArray();

                return packet;
            }

            public static FLAPPacket Ban(string screenname, uint fnacid)
            {
                FLAPPacket packet = new FLAPPacket();
                packet.Data = new byte[10 + 1 + screenname.Length];
                Tools.PutWord(packet.Data, 0, 0x000e);
                Tools.PutWord(packet.Data, 2, 0x0024);
                Tools.PutDword(packet.Data, 6, fnacid);
                packet.Data[10] = (byte)screenname.Length;
                Encoding.ASCII.GetBytes(screenname, 0, screenname.Length, packet.Data, 11);
                return packet;
            }

            public static FLAPPacket Unban(string screenname, uint fnacid)
            {
                FLAPPacket packet = new FLAPPacket();
                packet.Data = new byte[10 + 1 + screenname.Length];
                Tools.PutWord(packet.Data, 0, 0x000e);
                Tools.PutWord(packet.Data, 2, 0x0025);
                Tools.PutDword(packet.Data, 6, fnacid);
                packet.Data[10] = (byte)screenname.Length;
                Encoding.ASCII.GetBytes(screenname, 0, screenname.Length, packet.Data, 11);
                return packet;
            }

            public static FLAPPacket Close()
            {
                FLAPPacket packet = new FLAPPacket();
                packet.Data = new byte[10];
                Tools.PutWord(packet.Data, 0, 0x000e);
                Tools.PutWord(packet.Data, 2, 0x0023);
                return packet;
            }
        }
    }


}
