using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;

namespace Oscar5Plugin
{
    class RateClass
    {
        public ushort ClassID;
        public uint WindowSize;
        public uint ClearLevel;
        public uint AlertLevel;
        public uint LimitLevel;
        public uint DisconnectLevel;
        public uint CurrentLevel;
        public uint MaxLevel;
        public uint LastTime;
        public byte CurrentState;

        public RateCode RateCode;
        public RateState RateState;

        public Queue<FLAPPacket> PacketQueue;
        public FLAPSocket Socket;
        public Timer PacketTimer;
        public bool PacketTimerEnabled;
        public Timer ClearTimer;
        

        public RateClass(FLAPSocket socket)
        {
            Socket = socket;
        }

        private int GetPause()
        {
            switch (Socket.SocketType)
            {
                case FLAPSocketType.Auth:
                    return 0;
                case FLAPSocketType.BOS:
                    return 2100;
                case FLAPSocketType.Chat:
                    switch (ClassID)
                    {
                        case 0x0001:
                            return 2500;
                        case 0x0005:
                            return 7000;
                        default:
                            return 10000;
                    }
                default:
                    return 10000;
            }
        }

        public int CalculateTimeToLimitClear()
        {
            //this formula provided by openaimsprt1 on dev.aol.com forums
            //(clearValue*windowSize) - (currentAvg* (windowSize 1)) = delta

            return (int)((ClearLevel * WindowSize) - (CurrentLevel * (WindowSize - 1)));
        }

        public int CalculateTimeToAlertClear()
        {
            return (int)((AlertLevel * WindowSize) - (CurrentLevel * (WindowSize - 1)));
        }

        public void SetQueue()
        {
            
            if (PacketQueue == null)
            {
                PacketQueue = new Queue<FLAPPacket>();
            }
            lock (PacketQueue)
            {
                if (PacketTimer == null)
                {
                    PacketTimer = new Timer(new TimerCallback(PacketTimer_Callback));
                }

                if (ClearTimer == null)
                {
                    ClearTimer = new Timer(new TimerCallback(ClearTimer_Callback));
                }

                if (CurrentLevel < LimitLevel)
                {
                    RateState = RateState.Limited;

                    PacketTimerEnabled = true;

                    int disablewait = CalculateTimeToLimitClear();

                    int pause = GetPause();

                    PacketTimer.Change(disablewait, pause);

                    ClearTimer.Change(disablewait, Timeout.Infinite);
                }
                else if (CurrentLevel < AlertLevel)
                {
                    RateState = RateState.Alerted;

                    PacketTimerEnabled = true;

                    int disablewait = CalculateTimeToAlertClear();

                    int pause = GetPause();

                    PacketTimer.Change(pause, pause);

                    ClearTimer.Change(disablewait, Timeout.Infinite);
                }
                else
                {
                    RateState = RateState.None;
                }

            }
            
        }


        void PacketTimer_Callback(object state)
        {
            lock (PacketQueue)
            {
                if (PacketQueue.Count > 0)
                {
                    Socket.SendPacket(PacketQueue.Dequeue());
                }
                else
                {
                    if (RateState == RateState.None)
                    {
                        PacketTimer.Change(Timeout.Infinite, Timeout.Infinite);
                        PacketTimerEnabled = false;
                    }
                }
            }
        }

        void ClearTimer_Callback(object state)
        {
            lock (PacketQueue)
            {
                RateState = RateState.None;
            }
        }
    }
}
