using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using LighthouseShared;

namespace Uccom3ishParser
{
    public partial class PanelCCOMs : LighthouseShared.LHPanel
    {
        private SimpleCCOM m_CCOMPlugin;

        private CommandInfo m_SelectedCommand;

        public PanelCCOMs(SimpleCCOM ccomplugin)
        {
            InitializeComponent();
            this.PanelName = "CCOMs";
            this.m_CCOMPlugin = ccomplugin;
        }

        public override void UpdateUI()
        {
            base.UpdateUI();

            listCommands.Items.Clear();

            foreach (CommandInfo c in m_CCOMPlugin.m_Commands)
            {
                listCommands.Items.Add(new ListItem(String.Format("{0}.{1}", c.Plugin.Name, c.Name), c));
            }       
        }

        private void listCommands_SelectedIndexChanged(object sender, EventArgs e)
        {
            CommandInfo command = (CommandInfo)((ListItem)listCommands.SelectedItem).Tag;
            textName.Text = command.Name;
            textCommand.Text = command.Command;
            textPlugin.Text = command.Plugin.Name + " " + command.Plugin.Version.ToString();
            textHelp.Text = command.Help;
            checkEnabled.Checked = command.Enabled;

            m_SelectedCommand = command;
        }

        private void buttonSave_Click(object sender, EventArgs e)
        {
            textCommand.Text = textCommand.Text.ToLower().Replace(" ", "");
            if (textCommand.Text.Length < 1)
            {
                MessageBox.Show(this, "Command field cannot be blank.", "Error");
                return;
            }

            if (checkEnabled.Checked)
            {
                foreach (ICommandInfo c in m_CCOMPlugin.m_Commands)
                {
                    if (c != m_SelectedCommand && c.Command == textCommand.Text)
                    {
                        MessageBox.Show(this, "The command you specified is already assigned to another function. You must either change it, or disable it.", "Error");
                        return;
                    }
                }
            }

            m_SelectedCommand.m_Command = textCommand.Text;
            m_SelectedCommand.m_Help = textHelp.Text;
            m_SelectedCommand.m_Enabled = checkEnabled.Checked;
        }


    }

    class ListItem
    {
        private string m_ItemText;

        public object Tag;

        public ListItem(string itemtext, object tag)
        {
            this.m_ItemText = itemtext;
            this.Tag = tag;
        }

        public ListItem(string itemtext)
            : this(itemtext, null)
        {
        }

        public override string ToString()
        {
            return m_ItemText;
        }
    }

}

