using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using LighthouseShared;
using System.Text.RegularExpressions;
using System.IO;
using System.Xml;

namespace Uccom3ishParser
{
    public class SimpleCCOM : ICCOMPlugin, IPlugin
    {
        private IPluginHost m_PluginHost;

        internal List<CommandInfo> m_Commands;
        public IList<ICommandInfo> List_Commands
        {
            get
            {
                return (IList<ICommandInfo>)m_Commands;

            }
        }

        private FileStream m_SettingsFile;

        internal Dictionary<string, IUserInfo> m_Users;
        internal List<IUserInfo> m_UsersHere;

        internal List<ICommandInfo> m_DefaultPermissions;

        internal List<IUserInfo> m_Bans;

        #region ICommandParser Members

        public SimpleCCOM()
        {
            m_Commands = new List<CommandInfo>();
            m_Users = new Dictionary<string, IUserInfo>();
            m_UsersHere = new List<IUserInfo>();
            m_Bans = new List<IUserInfo>();

            m_DefaultPermissions = new List<ICommandInfo>();
        }

        public IDictionary<string, IUserInfo> List_Users
        {
            get { return m_Users; }
        }

        public IList<IUserInfo> List_UsersHere
        {
            get { return m_UsersHere; }
        }

        public IList<IUserInfo> List_Bans
        {
            get { return m_Bans; }
        }

        public string Name
        {
            get { return "SimpleCCOM"; }
        }

        public Version Version
        {
            get { return new Version("0.1"); }
        }

        public string Description
        {
            get { return "Command parser similar to Uccom3's"; }
        }

        public ICommandInfo RegisterCommand(string name, string defaultcommand, string help, IPlugin plugin, CommandCallback callback)
        {
            CommandInfo commandinfo = new CommandInfo(name, defaultcommand, help, plugin, callback);

            foreach (ICommandInfo c in m_Commands)
            {
                if (c.Command == commandinfo.Command)
                {
                    commandinfo.Enabled = false;
                    break;
                }
            }

            m_Commands.Add(commandinfo);

            

            return commandinfo;
        }

        public void UnregisterCommand(ICommandInfo commandinfo)
        {
            m_Commands.Remove((CommandInfo)commandinfo);
        }

        public void UnregisterCommandsByPlugin(IPlugin plugin)
        {
            CommandInfo[] tmpcmds = m_Commands.ToArray();

            foreach (CommandInfo c in tmpcmds)
            {
                if (c.Plugin == plugin)
                    m_Commands.Remove(c);
            }
        }

        #endregion


        #region IPlugin Members

        public void Load(IPluginHost host, System.IO.FileStream settingsfile)
        {
            m_SettingsFile = settingsfile;

            m_PluginHost = host;

            this.RegisterCommand("Help", "help", "Usage: {0} [command].\r\n Displays information about a command. With no argument, lists commands.", this, new CommandCallback(DoCCOMSingleArg));
            this.RegisterCommand("UserInfo", "info", "Usage: {0} [user].\r\n Displays information about a user. With no argument, displays info about yourself.", this, new CommandCallback(DoCCOMSingleArg));
            this.RegisterCommand("AddOp", "op", "Usage: {0} user.\r\n Adds a user to the op list and gives them the default permissions.", this, new CommandCallback(DoCCOMSplitArgs));
            this.RegisterCommand("RemoveOp", "unop", "Usage: {0} user.\r\n Removes a user from the op list.", this, new CommandCallback(DoCCOMSplitArgs));
            this.RegisterCommand("AddImmune", "imm", "Usage: {0} user.\r\n Adds a user to the immune list.", this, new CommandCallback(DoCCOMSplitArgs));
            this.RegisterCommand("RemoveImmune", "unimm", "Usage: {0} user.\r\n Removes a user from the immune list.", this, new CommandCallback(DoCCOMSplitArgs));
            this.RegisterCommand("Ban", "ban", "Usage: {0} user.\r\n Bans a user from the chat.", this, new CommandCallback(DoCCOMSplitArgs));
            this.RegisterCommand("Unban", "unban", "Usage: {0} user.\r\n Unbans a user from the chat.", this, new CommandCallback(DoCCOMSplitArgs));
            this.RegisterCommand("Kick", "kick", "Usage: {0} user.\r\n Kicks a user from the chat.", this, new CommandCallback(DoCCOMSplitArgs));
            this.RegisterCommand("SetPermissions", "setperms", "Usage: {0} user permissions.\r\n Sets the permissions of a user. Permissions should be given in a list separated by a space, with a + or - preceding each permission", this, new CommandCallback(DoCCOMSingleArg));
            this.RegisterCommand("Status", "status", "Usage: {0}.\r\n Displays status.", this, new CommandCallback(DoCCOMSingleArg));
            m_PluginHost.AddUIPanel(this, new PanelCCOMs(this));
            m_PluginHost.AddUIPanel(this, new PanelOps(this));
            m_PluginHost.AddUIPanel(this, new PanelBansMisc(this));
        }

        public void Unload()
        {
            m_SettingsFile.Close();
        }

        public void SaveSettings()
        {
            /*
            m_SettingsFile.Seek(0, SeekOrigin.Begin);
            m_SettingsFile.SetLength(0);

            StreamWriter sr = new StreamWriter(m_SettingsFile);

            sr.WriteLine("<settings>");

            foreach (IUserInfo u in m_Users)
            {
                sr.WriteLine("<user>");
                sr.WriteLine("");

            }
            */
        }

        #endregion

        void DoCCOMSingleArg(ICommandInfo commandinfo, IUserInfo userinfo, string args)
        {
             string normalized = args.ToLower().Replace(" ", "");
             switch (commandinfo.Name)
             {
                 case "Help":
                     if (normalized.Length < 1)
                     {
                         StringBuilder sb = new StringBuilder();
                         foreach (ICommandInfo c in userinfo.Permissions)
                         {
                             if (c.Enabled)
                                 sb.Append(String.Format("{0}, ", c.Command));
                         }
                         sb.Remove(sb.Length - 2, 2);

                         m_PluginHost.SendChatMessageByType("Command List", sb.ToString(), true);
                     }
                     else
                     {
                         foreach (ICommandInfo c in userinfo.Permissions)
                         {
                             if (c.Enabled && c.Command == normalized)
                             {
                                 m_PluginHost.SendChatMessageByType("Help", String.Format(c.Help, c.Command), true);
                                 return;
                             }
                         }
                         m_PluginHost.SendChatMessageByType("Error", String.Format("Command \"{0}\" does not exist.", normalized), false);
                     }
                     break;
                 case "UserInfo":
                     if (normalized.Length > 0)
                     {
                         string sn = normalized;
                         foreach (IUserInfo u in m_UsersHere)
                         {
                             if (sn == u.ScreenName || u.ScreenName.Contains(sn))
                             {
                                 sn = u.ScreenName;
                                 break;
                             }
                         }

                         if (m_Users.ContainsKey(sn))
                         {
                             IUserInfo u = m_Users[sn];



                             if ((u.ScreenName == normalized) || u.ScreenName.Contains(normalized))
                             {
                                 StringBuilder sb = new StringBuilder();
                                 if (u.Here)
                                 {
                                     sb.Append("Here, ");
                                 }
                                 if (u.Banned)
                                 {
                                     sb.AppendFormat("Banned (by {0}), ", u.BannedBy);
                                 }
                                 if (u.Immune)
                                 {
                                     sb.AppendFormat("Immune (by {0}), ", u.ImmunedBy);
                                 }
                                 if (u.Op)
                                 {
                                     try
                                     {
                                         sb.AppendFormat("Op (by {0}), ", u.OppedBy);

                                         sb.Append("Permissions: ");

                                         foreach (ICommandInfo c in u.Permissions)
                                         {
                                             sb.AppendFormat("{0} ", c.Command);
                                         }
                                     }
                                     catch (Exception)
                                     {
                                     }
                                 }

                                 m_PluginHost.SendChatMessageByType(String.Format("Info {0}", u.ScreenName), sb.ToString(), true);
                                 break;
                             }
                         }
                         else
                         {
                             m_PluginHost.SendChatMessageByType("Error", String.Format("Unable to view info: user {0} not found", sn), false);
                         }
                     }
                     break;
                 case "SetPermissions":
                     if (normalized.Length > 0)
                     {
                         string[] argsplit = args.ToLower().Split(" ".ToCharArray());

                         if (argsplit.Length < 2)
                         {
                             m_PluginHost.SendChatMessageByType("Error", "Unable to set permissions: not enough arguments", false);
                         }
                         else
                         {
                             argsplit[0] = CompleteSN(argsplit[0]);
                             if (m_Users.ContainsKey(argsplit[0]) && m_Users[argsplit[0]].Op)
                             {
                                 for (int i = 1; i < argsplit.Length; i++)
                                 {
                                     if (argsplit[i][0] == '-')
                                     {
                                         CCOMSetPermission(argsplit[0], argsplit[i].Substring(1), false);
                                     }
                                     else if (argsplit[i][0] == '+')
                                     {
                                         CCOMSetPermission(argsplit[0], argsplit[i].Substring(1), true);
                                     }
                                 }
                             }
                             else
                             {
                                 m_PluginHost.SendChatMessageByType("Error  Unable to set permissions", String.Format("{0} is not an op", argsplit[0]));
                             }
                         }
                     }
                     break;
                 case "Status":
                     if (true)
                     {
                         StringBuilder sb = new StringBuilder();

                         int optally = 0;
                         int immtally = 0;
                         int bantally = 0;

                         foreach (KeyValuePair<string, IUserInfo> kv in m_Users)
                         {
                             if (kv.Value.Op)
                                 optally++;
                             if (kv.Value.Immune)
                                 immtally++;
                             if (kv.Value.Banned)
                                 bantally++;

                         }

                         sb.AppendFormat("Op: {0} Immune: {1} Bans: {2}", optally, immtally, bantally);
                         sb.AppendFormat(" Owner: {0}", m_PluginHost.ChatInfoCCOM.Owner);

                         m_PluginHost.SendChatMessageByType("Status", sb.ToString(), true);
                     }
                     break;
                 default:
                     m_PluginHost.SendChatMessageByType("Error", "I dunno how to handle that command yet :(", false);
                     break;

             }
        }

        void DoCCOMSplitArgs(ICommandInfo commandinfo, IUserInfo userinfo, string args)
        {
            string[] argsplit = args.ToLower().Replace(" ", "").Split(",".ToCharArray());

            foreach (string arg in argsplit)
            {
                switch (commandinfo.Name)
                {
                    case "Ban":
                        foreach (string s in CompleteSNList(arg))
                        {
                            CCOMBan(s, userinfo.ScreenName);
                        }
                        break;
                    case "Unban":
                        bool unbanned = false;
                        foreach (IUserInfo u in m_Bans)
                        {
                            if (u.ScreenName.Contains(arg))
                            {
                                CCOMUnban(u.ScreenName);
                                unbanned = true;
                                break;
                            }
                        }
                        if (!unbanned)
                        {
                            CCOMUnban(arg);
                        }
                        break;
                    case "Kick":
                        foreach (string s in CompleteSNList(arg))
                        {
                            CCOMKick(s);
                        }
                        break;
                    case "AddOp":
                        CCOMAddOp(CompleteSN(arg), userinfo.ScreenName);
                        break;
                    case "RemoveOp":
                        CCOMRemoveOp(CompleteSN(arg));
                        break;
                    case "AddImmune":
                        CCOMAddImmune(CompleteSN(arg), userinfo.ScreenName);
                        break;
                    case "RemoveImmune":
                        CCOMRemoveImmune(CompleteSN(arg));
                        break;

                }
            }
        }

        /*
        void DoCCOM(ICommandInfo commandinfo, IUserInfo userinfo, string args)
        {
            string normalized = args.ToLower().Replace(" ", "");
            switch (commandinfo.Name)
            {
                case "Help":
                    if (normalized.Length < 1)
                    {
                        StringBuilder sb = new StringBuilder();
                        foreach (ICommandInfo c in userinfo.Permissions)
                        {
                            if (c.Enabled)
                                sb.Append(String.Format("{0}, ", c.Command));
                        }
                        sb.Remove(sb.Length - 2, 2);

                        m_PluginHost.SendChatMessageByType("Command List", sb.ToString(), true);
                    }
                    else
                    {
                        foreach (ICommandInfo c in userinfo.Permissions)
                        {
                            if (c.Enabled && c.Command == normalized)
                            {
                                m_PluginHost.SendChatMessageByType("Help", String.Format(c.Help, c.Command), true);
                                return;
                            }
                        }
                        m_PluginHost.SendChatMessageByType("Error", String.Format("Command \"{0}\" does not exist.", normalized), false);
                    }
                    break;
                case "Ban":
                    if (normalized.Length > 0)
                    {
                        foreach (IUserInfo u in m_UsersHere)
                        {
                            Console.WriteLine("normalized: {0}, screenname: {1}", normalized, u.ScreenName);
                            if (!u.Immune && (u.ScreenName == normalized || u.ScreenName.Contains(normalized)))
                            {
                                try
                                {
                                    m_PluginHost.SendBan(u.ScreenName);
                                }
                                catch (Exception ex)
                                {
                                    m_PluginHost.SendChatMessageByType("Error", String.Format("Unable to ban {0}: {1}", u.ScreenName, ex.Message), false);
                                    break;
                                }
                            }
                        }
                    }
                    break;
                case "UserInfo":
                    if (normalized.Length > 0)
                    {
                        string sn = normalized;
                        foreach (IUserInfo u in m_UsersHere)
                        {
                            if (sn == u.ScreenName || u.ScreenName.Contains(sn))
                            {
                                sn = u.ScreenName;
                                break;
                            }
                        }

                        if (m_Users.ContainsKey(sn))
                        {
                            IUserInfo u = m_Users[sn];

                            

                            if ((u.ScreenName == normalized) || u.ScreenName.Contains(normalized))
                            {
                                StringBuilder sb = new StringBuilder();
                                if (u.Here)
                                {
                                    sb.Append("Here, ");
                                }
                                if (u.Banned)
                                {
                                    sb.AppendFormat("Banned (by {0}), ", u.BannedBy);
                                }
                                if (u.Immune)
                                {
                                    sb.AppendFormat("Immune (by {0}), ", u.ImmunedBy);
                                }
                                if (u.Op)
                                {
                                    try
                                    {
                                        sb.AppendFormat("Op (by {0}), ", u.OppedBy);

                                        sb.Append("Permissions: ");

                                        foreach (ICommandInfo c in u.Permissions)
                                        {
                                            sb.AppendFormat("{0} ", c.Command);
                                        }
                                    }
                                    catch (Exception)
                                    {
                                    }
                                }

                                m_PluginHost.SendChatMessageByType(String.Format("Info {0}", u.ScreenName), sb.ToString(), true);
                                break;
                            }
                        }
                        else
                        {
                            m_PluginHost.SendChatMessageByType("Error", String.Format("Unable to view info: user {0} not found", sn), false);
                        }
                    }
                    break;
                case "SetPermissions":
                    if (normalized.Length > 0)
                    {
                        string[] argsplit = args.ToLower().Split(" ".ToCharArray());

                        if (argsplit.Length < 2)
                        {
                            m_PluginHost.SendChatMessageByType("Error", "Unable to set permissions: not enough arguments", false);
                        }
                        else
                        {
                            if (m_Users.ContainsKey(argsplit[0]))
                            {
                                IUserInfo u = m_Users[argsplit[0]];

                                if (!userinfo.Op)
                                {
                                    m_PluginHost.SendChatMessageByType("Error", "Unable to set permissions: user is not an op", false);
                                }
                                else
                                {
                                    for (int i = 1; i < argsplit.Length; i++)
                                    {
                                        string[] permsplit = argsplit[i].Split(":".ToCharArray());

                                        if (permsplit.Length == 2)
                                        {
                                            foreach (ICommandInfo c in m_Commands)
                                            {
                                                if (c.Enabled && c.Command == permsplit[0])
                                                {
                                                    if (permsplit[1] == "y")
                                                    {
                                                        try
                                                        {
                                                            u.Permissions.Add(c);
                                                        }
                                                        catch (Exception)
                                                        {
                                                        }
                                                    }
                                                    else if(permsplit[1] == "n")
                                                    {
                                                        try
                                                        {
                                                            u.Permissions.Remove(c);
                                                        }
                                                        catch (Exception)
                                                        {
                                                        }
                                                    }
                                                    break;
                                                }
                                            }
                                        }
                                        else
                                        {
                                            m_PluginHost.SendChatMessageByType("Error", "Unable to set permissions: invalid argument format", false);
                                        }
                                    }
                                    m_PluginHost.SendChatMessageByType("Set Permissions", String.Format("Permissions set for {0}", u.ScreenName), false);
                                }
                            }
                            else
                            {
                                m_PluginHost.SendChatMessageByType("Error", "Unable to set permissions: user not found", false);
                            }
                        }
                    }
                    break;
                case "RemoveImmune":
                    if (normalized.Length > 0)
                    {
                        UserInfo user = null;
                        foreach (IUserInfo u in m_UsersHere)
                        {
                            if (u.ScreenName == normalized || u.ScreenName.Contains(normalized))
                            {
                                if (u.Immune)
                                {
                                    user = (UserInfo)u;
                                    break;
                                }
                            }
                        }

                        if (user == null && m_Users.ContainsKey(normalized))
                        {
                            user = (UserInfo)m_Users[normalized];
                        }

                        if (user != null)
                        {
                            if (user.Immune)
                            {
                                user.m_Immune = false;
                                m_PluginHost.SendChatMessageByType("Remove Immune", user.ScreenName, false);
                                return;
                            }
                        }

                        m_PluginHost.SendChatMessageByType("Error", String.Format("Unable to remove immune: {0} is not immune", user.ScreenName), false);
                    }
                    break;
                case "Status":
                    if (true)
                    {
                        StringBuilder sb = new StringBuilder();

                        int optally = 0;
                        int immtally = 0;
                        int bantally = 0;

                        foreach (KeyValuePair<string, UserInfo> kv in m_Users)
                        {
                            if (kv.Value.Op)
                                optally++;
                            if (kv.Value.Immune)
                                immtally++;
                            if (kv.Value.Banned)
                                bantally++;

                        }

                        sb.AppendFormat("Op: {0} Immune: {1} Bans: {2}", optally, immtally, bantally);
                        sb.AppendFormat(" Owner: {0}", m_PluginHost.ChatInfoCCOM.Owner);

                        m_PluginHost.SendChatMessageByType("Status", sb.ToString(), true);
                    }
                    break;
                default:
                    m_PluginHost.SendChatMessageByType("Error", "I dunno how to handle that command yet :(", false);
                    break;
                
            }
        }
        */

        #region ICCOMPlugin Members

        public void HandleKickFailed(string screenname)
        {
            m_PluginHost.SendChatMessageByType("Error  Kick failed", screenname, false);
        }
        public void HandleKick(string screenname)
        {
            m_PluginHost.SendChatMessageByType("Kick", screenname, false);
        }

        public void HandleBan(string screenname)
        {
            UserInfo user = GetOrAddUser(screenname);

            user.m_Banned = true;
            m_Bans.Add(user);
            m_PluginHost.SendChatMessageByType("Ban", screenname, false);
        }

        public void HandleBanFailed(string screenname)
        {
            m_PluginHost.SendChatMessageByType("Error  Ban failed", screenname, false);
        }

        public void HandleUnban(string screenname)
        {
            if (m_Users.ContainsKey(screenname))
            {
                ((UserInfo)m_Users[screenname]).m_Banned = false;
                m_Bans.Remove(m_Users[screenname]);
            }
            m_PluginHost.SendChatMessageByType("Unban", screenname, false);
        }

        public void HandleUnbanFailed(string screenname)
        {
            m_PluginHost.SendChatMessageByType("Error  Unban failed", screenname, false);
        }

        public void HandleMessage(string screenname, string message)
        {
            screenname = screenname.ToLower().Replace(" ", "");
            if (!screenname.Equals(m_PluginHost.ClientCCOM.ScreenName) && m_Users.ContainsKey(screenname))
            {
                IUserInfo userinfo = m_Users[screenname];

                if (userinfo.Op)
                {
                    message = Regex.Replace(message, @"<(.|\n)*?>", string.Empty);

                    ICommandInfo commandinfo = null;

                    foreach (ICommandInfo c in m_Commands)
                    {
                        if (c.Enabled && message.Length >= c.Command.Length && message.Substring(0, c.Command.Length) == c.Command)
                        {
                            commandinfo = c;
                            break;
                        }
                    }

                    if (commandinfo != null)
                    {
                        if (userinfo.Permissions.Contains(commandinfo))
                        {
                            string args = message.Substring(message.IndexOf(commandinfo.Command) + commandinfo.Command.Length).TrimStart(" ".ToCharArray());
                            try
                            {
                                commandinfo.Callback(commandinfo, userinfo, args);
                            }
                            catch (Exception ex)
                            {
                                m_PluginHost.SendChatMessageByType("Error", String.Format("Exception while executing callback for {0}", commandinfo.Command), false);
                            }
                        }
                        else
                        {
                            m_PluginHost.SendChatMessageByType("Error", String.Format("{0} does not have permission to use {1}", screenname, commandinfo.Command), false);
                        }
                    }
                }
            }
        }

        public void HandleUserJoin(string screenname)
        {
            UserInfo user = GetOrAddUser(screenname);
            user.m_Here = true;
            try
            {
                m_UsersHere.Add(user);
            }
            catch (Exception)
            {
            }
            
        }

        public void HandleUserLeave(string screenname)
        {
            try
            {
                if (m_Users.ContainsKey(screenname))
                {
                    ((UserInfo)m_Users[screenname]).m_Here = false;
                    m_UsersHere.Remove(m_Users[screenname]);
                }
            }
            catch (Exception)
            {
            }
        }

        public void HandleChatJoin(IChatInfo chatinfo)
        {
            
        }

        public void HandleChatLeave(IChatInfo chatinfo, LeaveReason reason)
        {
            
        }

        internal UserInfo GetOrAddUser(string screenname)
        {
            screenname = screenname.ToLower().Replace(" ", "");

            if (m_Users.ContainsKey(screenname))
            {
                return (UserInfo)m_Users[screenname];
            }
            else
            {
                UserInfo userinfo = new UserInfo();
                userinfo.m_ScreenName = screenname;
                m_Users.Add(screenname, userinfo);
                return userinfo;
            }
        }

        internal UserInfo GetUser(string screenname)
        {
            screenname = screenname.ToLower().Replace(" ", "");

            if (m_Users.ContainsKey(screenname))
            {
                return (UserInfo)m_Users[screenname];
            }
            else
            {
                return null;
            }
        }

        internal string CompleteSN(string screenname)
        {
            foreach (UserInfo u in m_UsersHere)
            {
                if (u.ScreenName.Contains(screenname))
                    return u.ScreenName;
            }

            return screenname;
        }

        internal string[] CompleteSNList(string screenname)
        {
            List<string> snlist = new List<string>();

            foreach (UserInfo u in m_UsersHere)
            {
                if (u.ScreenName.Contains(screenname))
                    snlist.Add(u.ScreenName);
            }

            if (snlist.Count < 1)
                snlist.Add(screenname);

            return snlist.ToArray();
        }

        #endregion

        

        public void CCOMAddOp(string screenname, string oppedby)
        {
            UserInfo userinfo = GetOrAddUser(screenname);

            if (userinfo.Op)
            {
                m_PluginHost.SendChatMessageByType("Error  Cannot add op", String.Format("{0} is already op", userinfo.ScreenName), false);
            }
            else
            {
                userinfo.m_Op = true;
                userinfo.m_Permissions = new List<ICommandInfo>(m_DefaultPermissions);
                userinfo.m_OppedBy = oppedby;
                m_PluginHost.SendChatMessageByType("Add Op", userinfo.ScreenName, false);
            }
        }

        public void CCOMAddOp(string screenname)
        {
            CCOMAddOp(screenname, "?");
        }

        public void CCOMRemoveOp(string screenname)
        {
            if (m_Users.ContainsKey(screenname) && m_Users[screenname].Op)
            {
                UserInfo userinfo = (UserInfo)m_Users[screenname];
                userinfo.m_Op = false;
                userinfo.m_OppedBy = null;
                m_PluginHost.SendChatMessageByType("Remove Op", userinfo.ScreenName, false);
            }
            else
            {
                m_PluginHost.SendChatMessageByType("Error  Cannot remove op", String.Format("{0} is not an op", screenname), false);
            }
        }

        public void CCOMSetPermission(string screenname, string command, bool allowed)
        {
            if (m_Users.ContainsKey(screenname) && m_Users[screenname].Op)
            {
                StringBuilder sb = new StringBuilder();

                // i had some weird varible names here, but i fixed them cuz im a good boy ^_^
                UserInfo userinfo = (UserInfo)m_Users[screenname];
                foreach (CommandInfo c in m_Commands)
                {
                    if (c.Command == command)
                    {
                        if (allowed && !userinfo.m_Permissions.Contains(c))
                        {
                            userinfo.m_Permissions.Add(c);
                            sb.Append("+" + c.Command + " ");
                        }
                        else if (userinfo.m_Permissions.Contains(c))
                        {
                            userinfo.m_Permissions.Remove(c);
                            sb.Append("-" + c.Command + " ");
                        }

                        break;
                    }
                }

                m_PluginHost.SendChatMessageByType(String.Format("Set Permissions for {0}", userinfo.ScreenName), sb.ToString(), false);
            }
            else
            {
                m_PluginHost.SendChatMessageByType("Error  Cannot set permissions", String.Format("{0} is not an op", screenname), false);
            }
        }

        public void CCOMAddImmune(string screenname, string immunedby)
        {
            UserInfo userinfo = GetOrAddUser(screenname);

            if (!userinfo.Immune)
            {
                userinfo.m_Immune = true;
                userinfo.m_ImmunedBy = immunedby;
                m_PluginHost.SendChatMessageByType("Add Immune", userinfo.ScreenName, false);
            }
            else
            {
                m_PluginHost.SendChatMessageByType("Error", "Cannot add immune: {0} is already immune", false);
            }
        }

        public void CCOMAddImmune(string screenname)
        {
            CCOMAddImmune(screenname, "?");
        }

        public void CCOMRemoveImmune(string screenname)
        {
            if (m_Users.ContainsKey(screenname) && m_Users[screenname].Immune)
            {
                UserInfo userinfo = (UserInfo)m_Users[screenname];
                userinfo.m_Immune = false;
                userinfo.m_ImmunedBy = null;
                m_PluginHost.SendChatMessageByType("Remove Immune", userinfo.ScreenName, false);
            }
            else
            {
                m_PluginHost.SendChatMessageByType("Error", String.Format("Cannot remove immune: {0} is not immune", screenname), false);
            }
        }

        public void CCOMBan(string screenname, string bannedby)
        {
            if (m_Users.ContainsKey(screenname) && m_Users[screenname].Immune)
            {
                m_PluginHost.SendChatMessageByType("Error", String.Format("Unable to ban: {0} is immune", screenname), false);
            }
            else
            {
                GetOrAddUser(screenname).m_BannedBy = bannedby;
                m_PluginHost.SendBan(screenname);
            }
        }

        public void CCOMUnban(string screenname)
        {
            m_PluginHost.SendUnban(screenname);
        }

        public void CCOMKick(string screenname)
        {
            if (m_Users.ContainsKey(screenname) && m_Users[screenname].Immune)
            {
                m_PluginHost.SendChatMessageByType("Error", String.Format("Unable to kick: {0} is immune", screenname), false);
            }
            else
            {
                m_PluginHost.SendKick(screenname);
            }
        }

        public void ShowProperties()
        {

        }
    }
}
